jQuery.sap.require("sap.ui.model.Filter");
jQuery.sap.require("sap.ui.model.resource.ResourceModel");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.MessageDialog");
jQuery.sap.require("sas.risk.common.models.RiskJSONModel");
jQuery.sap.require("sas.risk.common.utils.LayoutService");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.stress.config.StressAPI");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.StressAPI");
jQuery.sap.require("sas.stress.wkbench.StressFixes");
jQuery.sap.require("sas.stress.wkbench.StressUtil");
jQuery.sap.require("sas.stress.wkbench.UserPrefs");

sap.ui.controller("sas.stress.config.view.ProcessDefinitions", {

    onInit: function (event) {
        var view = this.getView();
        var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.config.messagebundle" });
        view.setModel(i18N, "i18N");
        //console.log("**********ProcessDefinitions.onInit");
        //this.refresh();
        new sas.risk.common.utils.LayoutService(this.getView().table, "ProcessDefinitionsTable");

        sas.eventBus.subscribe(sas.stress.wkbench.EventBusConstants.CHANNEL_NAME, sas.stress.wkbench.EventBusConstants.PROCESS_DIAGRAM_DOUBLECLICKED, this.diagramDoubleClickHandler, this);

    },

    onBeforeRendering: function () {
        //console.log("**********ProcessDefinitions.onBeforeRendering");
        this.refresh();
    },

    refresh: function () {
        var controller = this;
        var view = this.getView();
        view.table.setBusyIndicatorDelay(0);
        view.table.setBusy(true);
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        view.table.setNoData(rb.getText("processDefinitionsTable.noData.loading.txt"));
        //console.log("**********ProcessDefinitions.refresh");

        $.when(sas.stress.wkbench.StressAPI.loadProcessDefinitions()).done(function (definitions) {
            // S1422197
            controller._displayInvalidProcessFixesWarning(definitions, rb);

            //console.log("**********ProcessDefinitions.refresh done loadProcessDefinitions ");
            if (view.table.getModel() instanceof sap.ui.model.json.JSONModel) {
                view.table.getModel().setData(definitions);
            } else {
                var definitionsModel = new sas.risk.common.models.RiskJSONModel(definitions);
                view.table.setModel(definitionsModel);

                //S1256573: Do sorting after creation in view so it won't overwrite LayoutService
                //var sorters = [new sap.ui.model.Sorter("name", false)];
                //view.table.getBinding("rows").sort(sorters);
            }
            view.table.setSelectedIndex(-1);

        }).fail(function(jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        }).always(function() {
            view.table.setBusy(false);
            view.table.setNoData(new sas.risk.common.widgets.ZeroStateLayout({
                icon: sas.icons.HC.ZEROSTATEADD,
                title: "{i18N>processDefinitionsTable.noData.title}",
                buttons: [
                    new sas.hc.m.Button({
                        text: "{i18N>processDefinitionsTable.noData.newButton.txt}",
                        press: $.proxy(controller.onImportExcel, controller)
                    }).addStyleClass("sasPrimaryButton")
                ]
            }));
        });
    },

    diagramDoubleClickHandler: function (channel, event, data) {

        var nodeData = data.nodeData;
        var view = this.getView();
        var that = this;

        if (!nodeData) {
            return;
        }

        view.diagramBreadCrumb.addBreadcrumbItem(nodeData.key, nodeData.text);

        this.loadDiagramSubProcess(nodeData.key);

    },

    loadDiagramSubProcess: function(nodeId) {

        var that = this;
        // SRINI: S1421522.. if the nodeId == "Root", we need to set it to undefined so that the process is invoked
        nodeId = (nodeId === "Root" ? undefined: nodeId);
        var view = that.getView();
        var selectedIndex = view.table.getSelectedIndex();
        var context = view.table.getContextByIndex(selectedIndex);

        if (nodeId !== undefined && context !== null) {
            var process = context.getObject();
            sas.stress.wkbench.StressAPI.loadSubDefinition(process.id, nodeId).done(function (json) {
                that.updateDiagram(json);
            });
        }
        else {
            that.onRowSelectionChanged();
        }

    },

    onRowSelectionChanged: function () {

        var view = this.getView();
        var selectedIndex = view.table.getSelectedIndex();
        var process;
        var context;

        if (selectedIndex !== -1) {

            context = view.table.getContextByIndex(selectedIndex);
            process = context.getObject();

            view.diagramBreadCrumb.removeAllItems();
            view.diagramBreadCrumb.addBreadcrumbItem("Root", process.name);

        }

        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        view.removeBtn.setEnabled(selectedIndex !== -1 &&  capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROCESS.DELETE));

        this.updateDiagram(process);

    },

    updateDiagram: function (process) {

        var view = this.getView();
        var splitter = view.splitter;

        if (view.getModel() === undefined) {
            var processModel = new sas.risk.common.models.RiskJSONModel({});
            view.setModel(processModel);
        }

        var model = view.getModel();

        if (!process) {

            model.setProperty("/process", null);
            model.setProperty("/process/name", null);

            splitter.removeContentArea(view.diagramPanel);
            splitter.addContentArea(view.zeroPanel);

        } else {

            splitter.removeContentArea(view.zeroPanel);
            splitter.addContentArea(view.diagramPanel);

            var syncPoints = {};
            var syncBars = {};
            var processData = {};
            processData.nodes = [];
            processData.links = [];

            _.each(process.nodes, function (node) {

                processData.nodes.push({
                    category: "MANUAL",
                    commentCount: 0,
                    icon: "NEUTRALSTATUS",
                    key: node.processNodeId,
                    label: "task.status.notstarted.txt",
                    taskGroup: node.taskGroup,
                    status: "DEFINED",
                    text: node.nodeLabel + " [" + node.processNodeId + "]"
                });

                if (node.nodeType === 1) {
                    hasEndNode = true;
                    processData.links.push({
                        to: node.processNodeId + "_END",
                        from: node.processNodeId
                    });
                    processData.nodes.push({key: node.processNodeId + "_END", text: "{i18N>task.activity.end.txt}", category: '_END_'});
                }
                if (node.nodeType === 0) {
                    hasStartNode = true;
                    processData.links.push({
                        to: node.processNodeId,
                        from: node.processNodeId + "_START"
                    });
                    processData.nodes.push({key: node.processNodeId + "_START", text: "{i18N>task.activity.start.txt}", category: '_START_'});
                }

                // Figure out where we need sync bars first
                _.each(node.predecessorNodes, function (linkNode) {
                    if (!syncPoints[node.processNodeId]) {
                        syncPoints[node.processNodeId] = {};
                    }
                    var syncCount = syncPoints[node.processNodeId][linkNode.syncGroup];
                    if (!syncCount) {
                        syncPoints[node.processNodeId][linkNode.syncGroup] = 0;
                    } else if (syncCount === 1) {
                        processData.nodes.push({
                            category: "_SYNCBAR_",
                            key: node.processNodeId + "_BAR"
                        });
                        processData.links.push({
                            to: linkNode.targetId,
                            from: node.processNodeId + "_BAR"
                        });
                        syncBars[node.processNodeId] = linkNode.syncGroup;
                    }
                    syncPoints[node.processNodeId][linkNode.syncGroup]++;
                });

                // Add all links and rewrite any that need to connect to sync bars
                _.each(node.predecessorNodes, function (linkNode) {
                    processData.links.push({
                        to: node.processNodeId + (syncBars[node.processNodeId] !== null && syncBars[node.processNodeId] === linkNode.syncGroup ? "_BAR" : ""),
                        from: linkNode.sourceId
                    });
                });

            });

            model.setProperty("/process", processData);
            model.setProperty("/process/name", process.name);

        }

    },

    onImportExcel: function () {
        var ctrl = this;
        var view = ctrl.getView();
        var dialog = sap.ui.getCore().byId("sas.stress.config.view.ImportProcessDefinitionsDialog");
        if (!dialog) {
            dialog = sap.ui.jsfragment("sas.stress.config.view.ImportProcessDefinitionsDialog", ctrl);
            var i18N = new sap.ui.model.resource.ResourceModel({ bundleName: "sas.stress.config.messagebundle" });
            dialog.setModel(i18N, "i18N");
            dialog.setModel(view.getModel("ctxt"), "ctxt");
        }

        dialog.open();
    },

    onFileUploaded: function (event) {
        var dialog = sap.ui.getCore().byId("sas.stress.config.view.ImportProcessDefinitionsDialog");

        if (this.busyDialog) {
            this.busyDialog.close();
        }

        dialog.close();

        var ctrl = this;
        var response = event.getParameter("responseRaw");
        if (response) {
            var responseJSON = JSON.parse(response);
            if (responseJSON.hasOwnProperty("errorCode")) {
                sas.stress.wkbench.StressUtil.handleServerFailure({response:responseJSON});
            } else {
                ctrl.refresh();
            }
        }
    },

    onTypeMissmatch: function (event) {
        var aFileTypes = event.getSource().getFileType();
        $.each(aFileTypes, function (key, value) {
            aFileTypes[key] = "*." + value;
        });
        var sSupportedFileTypes = aFileTypes.join(", ");
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        sas.hc.m.MessageDialog.error({text: rb.getText("fileUploadTypeMismatch.msg.fmt.txt", [event.getParameter("fileType"), sSupportedFileTypes]), title: rb.getText("importProcessDefinitionsDialog.window.title")});
    },

    onFileSizeExceed: function (event) {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        sas.hc.m.MessageDialog.error({text: rb.getText("fileUploadFileSizeExceed.msg.fmt.txt", [event.getSource().getMaximumFileSize()]), title: rb.getText("importProcessDefinitionsDialog.window.title")});
    },

    onDeleteProcessDefinition: function () {
        var view = this.getView();
        var controller = this;
        var table = view.table;
        var itemsSelected = table.getSelectedIndices();
        if (itemsSelected.length === 0) {
            return;
        }

        var processDefinitionList = [];
        //Get a list of process definition id's from the item list
        for (var i = 0; i < itemsSelected.length; i++) {
            var item = itemsSelected[i];

            // DD: get the id we need
            processDefinitionList[i] = table.getContextByIndex(item).getObject().id;
        }


        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var processDefnDeleteMsg = "";
        if (itemsSelected.length === 1) {
            var procObj = table.getContextByIndex(itemsSelected[0]).getObject();
            var processDefnName = procObj.name;
            processDefnDeleteMsg = rb.getText("processDefinition.delete.single.confirmation.fmt.txt", [processDefnName]);
        } else {
            processDefnDeleteMsg = rb.getText("processDefinition.delete.multiple.confirmation.fmt.txt", [itemsSelected.length]);
        }

        sas.hc.m.MessageDialog.warning({
            text: processDefnDeleteMsg,
            title: rb.getText("processDefinition.delete.window.title"),
            buttonActions: [sas.hc.m.MessageDialogAction.Delete, sas.hc.m.MessageDialogAction.Cancel],
            buttonPress: function (event) {
                var action = event.getParameter("origin").getProperty("messageDialogAction");
                if (action === sas.hc.m.MessageDialogAction.Delete) {
                    //Delete all the items in the itemsSelected array
                    sas.stress.config.StressAPI.deleteProcessDefinitions(processDefinitionList).done(function () {
                        controller.refresh();
                    }).fail(function (jqxhr) {
                        sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
                    });
                }
            }
        });

    },

    onImportBtnClick: function () {
        var rb = sap.ui.getCore().getLibraryResourceBundle("sas.stress.config");
        var view = this.getView();
        var fileUploader = sap.ui.getCore().byId("ImportProcessDefinitionsDialog.fileUploader");
        var attachment = fileUploader.oFileUpload.files.item(0);
        var definitionName = attachment.name.replace(/\.[^/.]+$/, "");
        var foundExisting = false;
        var that = this;
        $.each(view.table.getModel().getData(), function (index, definition) {
            if (definitionName === definition.name) {
                foundExisting = true;
            }
        });

        if (foundExisting) {
            sas.hc.m.MessageDialog.warning({
                text: rb.getText("processDefinition.overwrite.confirmation.fmt.txt", [definitionName]),
                title: rb.getText("processDefinition.overwrite.window.title"),
                buttonActions: [sas.hc.m.MessageDialogAction.Yes, sas.hc.m.MessageDialogAction.No],
                buttonPress: function (event) {
                    var action = event.getParameter("origin").getProperty("messageDialogAction");
                    if (action === sas.hc.m.MessageDialogAction.Yes) {
                        that.doDefinitionUpload(attachment);
                    }
                }
            });
        } else {
            this.doDefinitionUpload(attachment);
        }
    },

    layoutDiagram: function () {
        if (this.getView().workflowDiagram) {
            this.getView().workflowDiagram.layoutDiagram();
        }
    },

    doDefinitionUpload: function (attachment) {
        var ctrl = this;
        var dialog = sap.ui.getCore().byId("sas.stress.config.view.ImportProcessDefinitionsDialog");
        sas.stress.config.StressAPI.uploadProcessDefinitions(attachment).done(function () {
            ctrl.refresh();
            dialog.close();
        }).fail(function (jqxhr) {
            sas.stress.wkbench.StressUtil.handleServerFailure(jqxhr);
        });
    },

    filterProcessDefinitions: function (e) {

        var search = e.getParameter("value") || e.getParameter("newValue") || e.getParameter("query") || "";
        sas.stress.wkbench.UserPrefs.save({ 'sas.stress.config.processDefinitions_filter': search });

        var table = sap.ui.getCore().byId("processDefinitions-table");

        var filter1 = new sap.ui.model.Filter("name", sap.ui.model.FilterOperator.Contains, search);
        var filter2 = new sap.ui.model.Filter("description", sap.ui.model.FilterOperator.Contains, search);
        var filter3 = new sap.ui.model.Filter("createdByUser", sap.ui.model.FilterOperator.Contains, search);
        var allFilters = new sap.ui.model.Filter([filter1, filter2, filter3]);

        var binding = table.getBinding("rows");
        binding.filter(allFilters);
    },

    _updateSheetTemplate: function (sheetTemplate) {
        return sas.stress.config.StressAPI.updateSheetTemplate(sheetTemplate).done(function () {
        });
    },
    // S1422197
    _displayInvalidProcessFixesWarning: function(definitions, rb){
        var inValidDefnsFixed = "";
        _.each(definitions, function(definition) {
            if (definition.errors && definition.errors.length > 0){
                inValidDefnsFixed += rb.getText("list.bullet.txt");
                inValidDefnsFixed += definition.name;
                inValidDefnsFixed += "\n";
                _.each(definition.errors, function (error){
                    inValidDefnsFixed += rb.getText("process.definition.repaired.details.fmt", error);
                    inValidDefnsFixed += "\n";
                });
                inValidDefnsFixed += "\n";
            }
        });
        if (inValidDefnsFixed.length > 0){
            sas.hc.m.MessageDialog.warning({
                text: rb.getText("process.definitions.dialog.repaired.fmt", inValidDefnsFixed),
                title: rb.getText("process.definitions.dialog.title.txt")
            });
        }
    }
});