jQuery.sap.require("sap.ui.core.format.DateFormat");
jQuery.sap.require("sap.ui.core.Orientation");
jQuery.sap.require("sap.ui.model.Sorter");
jQuery.sap.require("sas.hc.m.Button");
jQuery.sap.require("sas.hc.m.Label");
jQuery.sap.require("sas.hc.m.SearchField");
jQuery.sap.require("sas.hc.m.Text");
jQuery.sap.require("sas.hc.m.ResponsiveToolbar");
jQuery.sap.require("sas.hc.m.ToolbarSpacer");
jQuery.sap.require("sas.hc.ui.commons.layout.BorderLayout");
jQuery.sap.require("sas.hc.ui.commons.layout.BorderLayoutArea");
jQuery.sap.require("sas.hc.ui.layout.Splitter");
jQuery.sap.require("sas.hc.ui.table.Table");
jQuery.sap.require("sas.hc.ui.table.Column");
jQuery.sap.require("sas.risk.common.widgets.WidgetFactory");
jQuery.sap.require("sas.risk.common.widgets.ZeroStateLayout");
jQuery.sap.require("sas.stress.wkbench.CapabilityManager");
jQuery.sap.require("sas.stress.wkbench.StressGoTemplates");
jQuery.sap.require("sas.stress.wkbench.widget.ProcessDiagram");

sap.ui.jsview("sas.stress.config.view.ProcessDefinitions", {

    getControllerName: function () {
        return "sas.stress.config.view.ProcessDefinitions";
    },

    createContent: function (controller) {
        this.setHeight("100%");

        this.table = this._createProcessDefinitionsTable(controller);

        this.diagramPanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
                topPadding: "1.5rem",
                content: this._createProcessPreview(controller).addStyleClass("processDefinitionDiagram")
            }
        );

        this.zeroPanel = sas.risk.common.widgets.WidgetFactory.buildPanel({
                topPadding: "1.5rem",
                content: new sas.risk.common.widgets.ZeroStateLayout({
                    icon: sas.icons.HC.ZEROSTATEALERT,
                    title: "{i18N>processPreview.zeroState.title}"
                }).addStyleClass("riskNoDataLayoutAreaBorder")
            }
        );

        this.splitter = new sas.hc.ui.layout.Splitter({
            contentAreas: [
                sas.risk.common.widgets.WidgetFactory.buildPanel({
                        bottomPadding: 0,
                        content: this.table
                    }
                ),
                this.zeroPanel
            ],
            orientation: sap.ui.core.Orientation.Vertical,
            width: "100%",
            height: "100%"
        });

        this.splitter.attachResize({}, function () {
            controller.layoutDiagram();
        });

        return this.splitter;
    },

    _createProcessPreview: function (controller) {
        var that = this;
        if (this.workflowDiagram) {
            return this.workflowDiagram;
        }//cache it

        var templates = new sas.stress.wkbench.StressGoTemplates();

        this.workflowDiagram = new sas.stress.wkbench.widget.ProcessDiagram({
            content: '<div style="width:100%; height:100%; min-height:200px;"></div>',
            id: 'PreviewProcessDiagram',
            preferDOM: false,
            options: {
                allowDelete: false,
                allowMove: false,
                allowLink: false,
                allowRelink: false,
                allowVerticalScroll: true,
                allowHorizontalScroll: true,
                hasHorizontalScrollbar: true,
                hasVerticalScrollbar: false,
                initialAutoScale: go.Diagram.UniformToFill,
                allowUndo: false,
                contentAlignment: go.Spot.Center,
                "toolManager.mouseWheelBehavior": go.ToolManager.WheelZoom,
                padding: 15
            },
            layout: templates.LAYOUT(),
            nodeTemplates: {
                "": templates.DEFAULT_TASK(),
                "_START_": templates.START_NODE(),
                "_END_": templates.END_NODE(),
                "_SYNCBAR_": templates.SYNC_BAR()
            },
            linkTemplates: {
                "": templates.LINK()
            },
            nodes: "{/process/nodes}",
            links: "{/process/links}"
        });

        var zoomInButton = new sas.hc.m.Button({
            icon: sas.icons.HC.ZOOMIN,
            tooltip: "{i18N>home.diagram.zoomInButton.txt}",
            press: function () {
                var commandHandler = that.workflowDiagram.getDiagram().commandHandler;
                if (commandHandler.canIncreaseZoom()) {
                    commandHandler.increaseZoom();
                }
            }
        });

        var zoomOutButton = new sas.hc.m.Button({
            icon: sas.icons.HC.ZOOMOUT,
            tooltip: "{i18N>home.diagram.zoomOutButton.txt}",
            press: function () {
                var commandHandler = that.workflowDiagram.getDiagram().commandHandler;
                if (commandHandler.canDecreaseZoom()) {
                    commandHandler.decreaseZoom();
                }
            }
        });

        var zoomFitButton = new sas.hc.m.Button({
            icon: sas.icons.HC.ZOOMTOFIT,
            tooltip: "{i18N>home.diagram.zoomFitButton.txt}",
            press: function () {
                var commandHandler = that.workflowDiagram.getDiagram().commandHandler;
                commandHandler.zoomToFit();
            }
        });

        this.diagramBreadCrumb = new sas.hc.ui.commons.breadcrumb.Breadcrumb({
            itemSelected: function(event) {
                // S1419643
                var selected = event.getParameter("item");
                controller.loadDiagramSubProcess(selected ? selected.getProperty("key") : undefined);
            }
        });

        var zoomToolbar = new sas.risk.common.layout.BarLayout({
            content: [
                this.diagramBreadCrumb,
                new sas.risk.common.layout.BarLayoutSpacer,
                zoomInButton,
                zoomOutButton,
                zoomFitButton
            ]
        }).addStyleClass("hierarchyDiagramToolbar");

        var workflowLayout = new sas.hc.ui.commons.layout.BorderLayout({ width: "100%", height: "100%" });
        var headerArea = new sas.hc.ui.commons.layout.BorderLayoutArea({
            size: "2.5rem",
            content: zoomToolbar
        }).addStyleClass("diagramToolbar");

        this.diagramArea = new sas.hc.ui.commons.layout.BorderLayoutArea({ content: this.workflowDiagram }).addStyleClass("diagramContainer");

        workflowLayout.setTop(headerArea);
        workflowLayout.setCenter(this.diagramArea);

        return workflowLayout;
    },

    _createTableHeader: function (controller) {
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        var importExcelBtn = new sas.hc.m.Button({
            icon: sas.icons.HC.IMPORT,
            enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROCESS.IMPORT),
            press: $.proxy(controller.onImportExcel, controller),
            tooltip: "{i18N>processDefinitions.importExcelBtn.tip}"
        });

        this.removeBtn = new sas.hc.m.Button({
            icon: sas.icons.HC.DELETE,
            enabled: false,
            press: $.proxy(controller.onDeleteProcessDefinition, controller),
            tooltip: "{i18N>processDefinitions.removeBtn.tip}"
        });

        return new sas.hc.m.ResponsiveToolbar({
            content: [
                this._createSearchField(controller),
                new sas.hc.m.ToolbarSpacer(),
                importExcelBtn,
                new sas.hc.m.Button({
                    icon: sas.icons.HC.REFRESH,
                    tooltip: "{i18N>button.refresh.action.label}",
                    enabled: capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROCESS.READ),
                    press: $.proxy(controller.refresh, controller)
                }),
                this.removeBtn
            ]
        });
    },

    _createSearchField: function (controller) {
        return new sas.hc.m.SearchField({
            id: "ProcessDefinitions-searchFld",
            liveChange: $.proxy(controller.filterProcessDefinitions, controller),
            search: $.proxy(controller.filterProcessDefinitions, controller),
            filterMode: true
        });
    },

    _createProcessDefinitionsTable: function (controller) {
        var table = new sas.hc.ui.table.Table({
            id: "processDefinitions-table",
            showTableOptions: false,
            visibleRowCountMode: sas.hc.ui.table.VisibleRowCountMode.Auto,
            minAutoRowCount: 1,
            selectionBehavior: sas.hc.ui.table.SelectionBehavior.Row,
            selectionMode: sas.hc.ui.table.SelectionMode.Single,
            rowSelectionChange: $.proxy(controller.onRowSelectionChanged, controller),
            noData: "{i18N>processDefinitionsTable.noData.loading.txt}",
            toolbar: this._createTableHeader(controller)
        });

        table.addColumn(new sas.hc.ui.table.Column({
            id: "processDefinitions-nameColumn",
            label: new sas.hc.m.Label({ text: "{i18N>processDefinitionsTable.nameColumn.header.title}" }),
            template: new sas.hc.m.Text({ wrapping: false, tooltip: "{name}" }).bindProperty("text", "name"),
            sortProperty: "name",
            width: "15rem"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id: "processDefinitions-descColumn",
            label: new sas.hc.m.Label({ text: "{i18N>processDefinitionsTable.descriptionColumn.header.title}" }),
            template: new sas.hc.m.Text({ wrapping: false, tooltip: "{description}" }).bindProperty("text", "description"),
            sortProperty: "description"
            //width: "25%"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id: "processDefinitions-createdByColumn",
            label: new sas.hc.m.Label({ text: "{i18N>processDefinitionsTable.createdByColumn.header.title}" }),
            template: new sas.hc.m.Text({ wrapping: false, tooltip: "{createdByUser}" }).bindProperty("text", "createdByUser"),
            sortProperty: "createdByUser"
            //width: "25%"
        }));

        table.addColumn(new sas.hc.ui.table.Column({
            id: "processDefinitions-createdOnColumn",
            label: new sas.hc.m.Label({ text: "{i18N>processDefinitionsTable.createdOnColumn.header.title}" }),
            template: new sas.hc.m.Text({
                wrapping: false,
                tooltip: {
                    path: "createdTime",
                    formatter: function (dttm) {
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                },
                text: {
                    path: "createdTime",
                    formatter: function (dttm) {
                        var dateTimeFormat = sap.ui.core.format.DateFormat.getDateTimeInstance();
                        return dateTimeFormat.format(new Date(dttm));
                    }
                }
            }),//.bindProperty("text", "createdTime"),
            sortProperty: "createdTime"
            //width: "25%"
        }));

        table.bindRows("/");

        //S1256573: Default sort by Name asc.
        var bindingInfo = table.getBindingInfo("rows");
        if (bindingInfo) {
            var sorters = [new sap.ui.model.Sorter("name", false)];
            bindingInfo.sorter = sorters;
        }
        return table;
    },

    onRowSelectionChanged: function (e) {
        var hasSelections = (this.table.getSelectedIndices().length > 0);
        var capabilityManager = sas.stress.wkbench.CapabilityManager.getInstance();
        this.removeBtn.setEnabled(hasSelections && capabilityManager.isCapableOf(sas.stress.wkbench.CapabilityManager.PROCESS.DELETE));
    },

    attachSelectionChanged: function (func, listener) {
        this.attachEvent(this.getId() + "_SELECTION_CHANGED", func, listener);
    },

    detachSelectionChanged: function (func, listener) {
        this.detachEvent(this.getId() + "_SELECTION_CHANGED", func, listener);
    },

    destroy: function () {
        sap.ui.core.mvc.JSView.prototype.destroy.apply(this, arguments);
    }
});
