jQuery.sap.declare("sas.stress.wkbench.parser.ExpressionUtil");
jQuery.sap.require("sas.stress.wkbench.parser.TokenType");
jQuery.sap.require("sap.ui.core.format.NumberFormat");
jQuery.sap.require("sas.stress.wkbench.StressUtil");

sas.stress.wkbench.parser.ExpressionUtil={
		print:function(expr,fnContext,builder){
			var ExpressionUtil=sas.stress.wkbench.parser.ExpressionUtil;
			var TokenType=sas.stress.wkbench.parser.TokenType;
			var isRoot=(builder===undefined);
			switch(expr.type){
			case "CALL":
				if(!builder){
					builder='';
				}
				builder=ExpressionUtil.print(expr.func,fnContext,builder);
				builder+='(';
				for(var i=0;i<expr.args.length;i++){
					builder=ExpressionUtil.print(expr.args[i],fnContext,builder);
					if(i<expr.args.length-1){
						builder+=sas.stress.wkbench.StressUtil.whatListSeparator()/*', '*/;
					}
				}
				return builder+=')';
			case "COMPARISON":
				builder=isRoot?'':builder+'(';
				builder=ExpressionUtil.print(expr.left,fnContext,builder);
				builder+=' '+TokenType[expr.comparator].punctuator+' ';
			    builder=ExpressionUtil.print(expr.right,fnContext,builder);
				return builder+((isRoot)?'':')');
			case "CONDITIONAL":
				builder=isRoot?'':builder+'(';
				builder=ExpressionUtil.print(expr.condition,fnContext,builder);
				builder+=' ? ';
				builder=ExpressionUtil.print(expr.thenArm,fnContext,builder);
				builder+=' : ';
				builder=ExpressionUtil.print(expr.elseArm,fnContext,builder);
				return builder+((isRoot)?'':')');
			case "NAME":
				if(!builder){
					builder='';
				}
				if(fnContext){
					switch(ExpressionUtil._getType(expr.name)){
						case "STRING":
							return builder+expr.name;
						case "NUMBER":
							var num=expr.name;
							if (num.indexOf(".")>=0) {
								num = sas.stress.wkbench.StressUtil.replaceAll(num,".", sas.stress.wkbench.StressUtil.whatDecimalSeparator());
							}
							return builder+num;
						//break;
						case "BOOLEAN":
							return builder+expr.name.toUpperCase();
						//break;
						default:
							return builder+fnContext(expr.name);
					}
				}
				return builder+expr.name;
			case "OPERATOR":
				builder=isRoot?'':builder+'(';
				builder=ExpressionUtil.print(expr.left,fnContext,builder);
				builder+=' '+TokenType[expr.operator].punctuator+' ';
				builder=ExpressionUtil.print(expr.right,fnContext,builder);
				return builder+((isRoot)?'':')');
			case "POSTFIX":
				builder=isRoot?'':builder+'(';
				builder=ExpressionUtil.print(expr.left,fnContext,builder);
				return builder+TokenType[expr.operator].punctuator+((isRoot)?'':')');
			case "PREFIX":
				var skipParens=(builder && builder.indexOf(", ",builder.length-2)!==-1);//skip parenthesis if it is a function call arg
				builder=isRoot?'':builder+(skipParens?'':'(');
				builder+=TokenType[expr.operator].punctuator;
				builder=ExpressionUtil.print(expr.right,fnContext,builder);
				return builder+((isRoot||skipParens)?'':')');
			}
		},
		toString:function(expr,tab){
			var ExpressionUtil=sas.stress.wkbench.parser.ExpressionUtil;
			var TokenType=sas.stress.wkbench.parser.TokenType;
			var tab=tab?tab:0;
			switch(expr.type){
			case "CALL":
				var buff='CALL:';
				buff+=ExpressionUtil._tab(tab)+'|--(FUNCTION)'+ExpressionUtil.toString(expr.func,tab+1);
				for(var i=0;i<expr.args.length;i++){
					buff+=ExpressionUtil._tab(tab)+'|--(ARG['+i+'])'+ExpressionUtil.toString(expr.args[i],tab+1);
				}
				return buff;
			case "COMPARISON":
				var buff='COMPARISON:'+TokenType[expr.comparator].punctuator;
				buff+=ExpressionUtil._tab(tab)+'|--(LEFT)'+ExpressionUtil.toString(expr.left,tab+1);
				buff+=ExpressionUtil._tab(tab)+'|--(RIGHT)'+ExpressionUtil.toString(expr.right,tab+1);
				return buff;
			case "CONDITIONAL":
				var buff='CONDITIONAL:';
				buff+=ExpressionUtil._tab(tab)+'|--(CONDITION)'+ExpressionUtil.toString(expr.condition,tab+1);
				buff+=ExpressionUtil._tab(tab)+'|--(THEN)'+ExpressionUtil.toString(expr.thenArm,tab+1);
				buff+=ExpressionUtil._tab(tab)+'|--(ELSE)'+ExpressionUtil.toString(expr.elseArm,tab+1);
				return buff;
			case "NAME":
				var buff='NAME:'+expr.name;
				return buff;
			case "OPERATOR":
				var buff='OPERATOR:'+TokenType[expr.operator].punctuator;
				buff+=ExpressionUtil._tab(tab)+'|--'+ExpressionUtil.toString(expr.left,tab+1);
				buff+=ExpressionUtil._tab(tab)+'|--'+ExpressionUtil.toString(expr.right,tab+1);
				return buff;
			case "POSTFIX":
				var buff='POST-OPERATOR:';
				buff+=ExpressionUtil._tab(tab)+'|--(VALUE)'+ExpressionUtil.toString(expr.left,tab+1);
				buff+=ExpressionUtil._tab(tab)+'|--(OPERATOR)'+TokenType[expr.operator].punctuator;
				return buff;
			case "PREFIX":
				var buff='PRE-OPERATOR:'+TokenType[expr.operator].punctuator;
				buff+=ExpressionUtil._tab(tab)+'|--'+ExpressionUtil.toString(expr.right,tab+1);
				return buff;
			}
		},
		evaluate:function(expr,fnContext,ignore){
			var ExpressionUtil=sas.stress.wkbench.parser.ExpressionUtil;
			var TokenType=sas.stress.wkbench.parser.TokenType;
			switch(expr.type){
			case "CALL":
				var func;
				if (ignore===undefined) {
					func=ExpressionUtil.evaluate(expr.func, fnContext);
				}
				var values=[];
				for(var i=0;i<expr.args.length;i++){
					if (ignore===undefined) {
						values.push(ExpressionUtil.evaluate(expr.args[i], fnContext));
					}else{
						var evaluatedValues=[];
						evaluatedValues.push(ExpressionUtil.evaluate(expr.args[i], fnContext));
						var saveItUp=function(num){
							if($.isArray(num)){
								_.each(num,saveItUp);
							}else if(_.isNumber(num)){
								values.push(num);
							}else{
								values.push(0.0);	// not an array or number
							}
						};
						_.each(evaluatedValues, saveItUp);
					}
				}
				if (ignore===undefined) {
					return func.apply(null, values);
				}else{
					return values;
				}
			case "COMPARISON":
				var leftValue=ExpressionUtil.evaluate(expr.left,fnContext);
				var rightValue=ExpressionUtil.evaluate(expr.right,fnContext);
				if (_.isNumber(leftValue)&&_.isNumber(rightValue)){	// S1401405
					leftValue=sas.stress.wkbench.StressUtil.precisionRound(leftValue);
					rightValue=sas.stress.wkbench.StressUtil.precisionRound(rightValue);
				}
				switch(expr.comparator){
					case TokenType.EQUAL.name:
						return leftValue===rightValue;
					case TokenType.NOT_EQUAL.name:
						return leftValue!==rightValue;
					case TokenType.GREATER_THAN.name:
						return leftValue>rightValue;
					case TokenType.GREATER_THAN_EQUAL.name:
						return leftValue>=rightValue;
					case TokenType.LESS_THAN.name:
						return leftValue<rightValue;
					case TokenType.LESS_THAN_EQUAL.name:
						return leftValue<=rightValue;
				}
			case "CONDITIONAL":
				var conditionValue = ExpressionUtil.evaluate(expr.condition,fnContext);
				if(conditionValue===true){
					return ExpressionUtil.evaluate(expr.thenArm,fnContext);
				} else {
					return ExpressionUtil.evaluate(expr.elseArm,fnContext);
				}
			case "NAME":
				switch(ExpressionUtil._getType(expr.name)){
				case "STRING":
					return expr.name.substring(1,expr.name.length-1);
				case "BOOLEAN":
					return expr.name.toUpperCase()==="TRUE";
				case "REFERENCE":
					return fnContext(expr.name);
				case "NUMBER":
				default:
					return parseFloat(expr.name);
				}
			case "OPERATOR":
				var leftValue=ExpressionUtil.evaluate(expr.left,fnContext);
				var rightValue=ExpressionUtil.evaluate(expr.right,fnContext);
				if(leftValue===null){
					leftValue=0;
				}
				if(rightValue===null){
					rightValue=0;
				}
				if(!_.isNumber(leftValue) || !_.isNumber(rightValue)){
					throw ("#VALUE!");
				}/*else{
					leftValue=Number(leftValue);
					rightValue=Number(rightValue);
				}*/
				switch(expr.operator){
					case TokenType.PLUS.name:
						return leftValue+rightValue;
					case TokenType.MINUS.name:
						return leftValue-rightValue;
					case TokenType.ASTERISK.name:
						return leftValue*rightValue;
					case TokenType.SLASH.name:
						return leftValue/rightValue;
					case TokenType.CARET.name:
						return Math.pow(leftValue,rightValue);
				}
			case "POSTFIX":
				var leftValue=ExpressionUtil.evaluate(expr.left,fnContext);
				switch(expr.operator){
					case TokenType.BANG.name:
						//do factorial here.. not needed right now.
					break;
					case TokenType.PERCENTAGE.name:
						return (leftValue*1)/100;
				}
			case "PREFIX":
				var rightValue=ExpressionUtil.evaluate(expr.right,fnContext);
				switch(expr.operator){
					case TokenType.MINUS.name:
						return -rightValue;
					case TokenType.PLUS.name:
						return +rightValue;
					case TokenType.BANG.name:
						return !rightValue;
				}
			}
		},
		collect:function(expr,type){
			var ExpressionUtil=sas.stress.wkbench.parser.ExpressionUtil;
			var TokenType=sas.stress.wkbench.parser.TokenType;
			var hits=[];
			if(expr.type===type){
				hits.push(expr);
			}
			switch(expr.type){
			case "CALL":
				for(var i=0;i<expr.args.length;i++){
					hits=hits.concat(ExpressionUtil.collect(expr.args[i],type));
				}
				return hits;
			case "COMPARISON":
				hits=hits.concat(ExpressionUtil.collect(expr.left,type));
				hits=hits.concat(ExpressionUtil.collect(expr.right,type));
				return hits;
			case "CONDITIONAL":
				hits=hits.concat(ExpressionUtil.collect(expr.condition,type));
				hits=hits.concat(ExpressionUtil.collect(expr.thenArm,type));
				hits=hits.concat(ExpressionUtil.collect(expr.elseArm,type));
				return hits;
			case "NAME":
				return hits;
			case "OPERATOR":
				hits=hits.concat(ExpressionUtil.collect(expr.left,type));
				hits=hits.concat(ExpressionUtil.collect(expr.right,type));
				return hits;
			case "POSTFIX":
				hits=hits.concat(ExpressionUtil.collect(expr.left,type));
				return hits;
			case "PREFIX":
				hits=hits.concat(ExpressionUtil.collect(expr.right,type));
				return hits;
			}
		},
		create:function(type,arg1,arg2,arg3){
			var ExpressionUtil=sas.stress.wkbench.parser.ExpressionUtil;
			var TokenType=sas.stress.wkbench.parser.TokenType;
			var expr={type:type};
			switch(type){
			case "CALL":
				expr.func=arg1;
				expr.args=arg2;
				return expr;
			case "COMPARISON":
				expr.left=arg1;
				expr.comparator=arg2.name;
				expr.right=arg3;
				return expr;
			case "CONDITIONAL":
				expr.condition=arg1;
				expr.thenArm=arg2;
				expr.elseArm=arg3;
				return expr;
			case "NAME":
				expr.name=arg1;
				return expr;
			case "OPERATOR":
				expr.left=arg1;
				expr.operator=arg2.name;
				expr.right=arg3;
				return expr;
			case "POSTFIX":
				expr.left=arg1;
				expr.operator=arg2.name;
				return expr;
			case "PREFIX":
				expr.operator=arg1.name;
				expr.right=arg2;
				return expr;
			}
		},
		_getType:function(name){
			if(isNaN(name)){
				if(sas.stress.wkbench.parser.ExpressionUtil._isQuoted(name)){
					return "STRING";
				}
				var uppCase=name.toUpperCase();
				if(uppCase==="TRUE"||uppCase==="FALSE"){
					return "BOOLEAN";
				}
				return "REFERENCE";
			} else {
				return "NUMBER";
			}
		},
		_isQuoted:function(str){
			return str.length>=2 &&
				  (str[0]==='"' || str[0]==="'") &&
				  (str[str.length-1]==="'" || str[str.length-1]==='"');
		},
		_tab:function(size){
			if(!size){
				size=0;
			}
			var tab='\t';
			for(var i=0;i<size;i++){
				tab+='\t';
			}
			return '\n'+tab;
		}
};