---
--    Copyright (c) 2016 by SAS Institute Inc., Cary, NC, USA.
-- 
--- functions for manipulating / moving files
--  
--  
-- @module file_utils
local M={}
require 'risk.common.global_init'
local utils    = require 'risk.common.rest_utils'
--- Copies files from one folder to another. Recursive.
-- @param from_folder - The name of the folder to copy files from
-- @param to_folder - The name of the folder to copy files to
-- @param file_sep - The file separator, '/' or '\'
-- @param recursive - boolean, whether or not to go into folders and copy those files as well
-- @param index - DO NOT USE!!! This is internally used to assign unique filerefs to the files and directories.
-- @return file_list - an integer-indexed array with the names of the files thatw ere copied
--NOTE: This is all done in a datastep and uses the file statement to create the files
--      before writing to them. To write to the log, you need to issue a 'file log' statement first.
--      If you modify this code, just be sure you always use a file statement before a put to be sure you
--      are writing to the location you think you are.
function M.folder_copy(from_folder, to_folder, file_sep, recursive, index)
   if not recursive then recursive = false end
   
   local file_list = {}
   if not index then index = 1 end
   
   --First check to make sure the folders exist
   local dest = sasxx.assign(to_folder,"dest")
   local exists = dest:exists()
   if not exists then
      dest:deassign()
--       sas.print("%3z The destination folder does not exist - creating it.")
      sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_folder_copy_1', 'NOQUOTE'))
      local path = sasxx.mkdir(to_folder)
      dest = sasxx.assign(to_folder,"dest")
      exists = dest:exists()
      if not exists then
--          sas.print("%1z Was not able to create the directory "..to_folder)
         sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_folder_copy_2', 'NOQUOTE'),to_folder)
         return nil
      end
   end
   dest:deassign()
   
   local from = sasxx.assign(from_folder,"from"..index)
   exists = from:exists()
   if not exists then
--       sas.print("%1z The source folder does not exist - can't perform the copy")
      sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_folder_copy_3', 'NOQUOTE'))
      return nil
   end   
   if not from then 
--       sas.print("%1zCould not assign the fileref 'from"..index.."' to "..from_folder)
      sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_folder_copy_4', 'NOQUOTE'),from_folder)
      return 
   end
   local i = 0
   local from_dir = from:opendir()
   for dir_ent in from_dir:read() do
      if dir_ent.name ~= '.' and dir_ent.name ~= '..' then
         local entry = sasxx.assign(from_folder.."/"..dir_ent.name)
         local ent_dir = entry:opendir()
         if not ent_dir then 
            i = i + 1
            file_list[i] = dir_ent.name
            Debug("Copying:")
            Debug(from_folder.."/"..dir_ent.name)
            Debug("TO:")
            Debug(to_folder.."/"..dir_ent.name)
            local to_file_log = sasxx.assign(to_folder..file_sep..dir_ent.name,"tf"..index)
            local from_file_log = sasxx.assign(from_folder..file_sep..dir_ent.name,"ff"..index)
            local to_file =  to_file_log:open('w','b')
            local from_file =  from_file_log:open('r','b')
            for line in from_file:lines() do
               to_file:write(line)
            end
            from_file:close()
            to_file:close()
            to_file_log:deassign()
            from_file_log:deassign()
         else
            ent_dir:close()
            if recursive then 
               local path = sasxx.mkdir(dir_ent.name, to_folder)
               Debug("Creating the directory "..dir_ent.name.." in foler "..to_folder)
               M.folder_copy(from_folder..file_sep..dir_ent.name,to_folder..file_sep..dir_ent.name,file_sep,true,index+1)  
            end
         end
         entry:deassign()    
      end
   end
   from_dir:close()
   from:deassign()
   return(file_list)
end

function M.file_copy(from,to)
   local tf_log = sasxx.assign(to,"tf")
   if not tf_log then
      sas.print("%1zCould not assign a fileref to %s",tostring(to))
      return
   end
   local ff_log = sasxx.assign(from,"ff")
   if not ff_log then
      sas.print("%1zCould not assign a fileref to %s",tostring(from))
      return
   end   
   local to_file =  tf_log:open('w','b')
   if not to_file then
      sas.print("%1zCould not open %s",tostring(to_file))
      return
   end   
   local from_file =  ff_log:open('r','b')
   if not from_file then
      sas.print("%1zCould not open %s",tostring(from_file))
      return
   end    
   for line in from_file:lines() do
      to_file:write(line)
   end
   from_file:close()
   to_file:close()
   tf_log:deassign()
   ff_log:deassign()
end

return M
