--- Business rules Manager interface
-- Need to provide the following SAS macros
-- ruleFlowName - The name of the rule flow you want to use
-- version - The version of the rule flow - 1.0  1.1 
-- Source - Source data set - sasuser.foo
-- Dest - Destination data  - work.other
local rest = require 'risk.stress.rest_service'
local JSON = require 'risk.common.json'
local metadata = require 'risk.common.metadata'
JSON.strictTypes = true
local begintime = sas.time()
 -- First get the rest api information from metadata
 -- Actually need SASBusinessRulesManagementWeb
   local hostname, port, protocol, appName, URL = metadata.get_rest_end_point_dtls("SASRiskandFinanceWorkbench REST")
   if not URL then
--       sas.print("%1zWas unable to get the metadata information - check that the server is running and that you have the proper permissions.")
      sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_1', 'NOQUOTE'))
      return
   end
URL = protocol .."://" .. hostname ..":"..port .."/SASBusinessRulesManagerWeb/rest/"
rest.set_url(URL)
local pass, json = rest.request( "get", "ruleFlows", nil , "Accept:application/vnd.sas.businessrules.flows+json;version=1")
if not pass then
--    sas.print("%1zCan't connect to %s.",  URL)
   sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_2', 'NOQUOTE'),  URL)
   return
end
local ruleFlows= JSON:decode(json)
Debug(table.tostring(ruleFlows))
-- get the id
local ruleFlowName = sas.symget("ruleFlowName")
if ruleFlowName == nil then
--    sas.print("%1zNo rule flow name provided.")   
   sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_3', 'NOQUOTE'))
   return
end
local ruleFlowId
for i, flow in ipairs(ruleFlows.ruleflows) do
    if flow.name == ruleFlowName then
	    ruleFlowId = flow.id
		break
	end  
end
if ruleFlowId == nil then 
--    sas.print("%1zCan't find selected rule flow " .. ruleFlowName .. ".")
   sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_4', 'NOQUOTE'), ruleFlowName)
   return;
end
 pass, json = rest.request( "get", "ruleFlows/".. ruleFlowId.."/deployments", nil , "Accept:application/vnd.sas.businessrules.deployments+json;version=1")
if not pass then
--    sas.print("%1zCan't find a published version of the rule flow:%s.", ruleFlowName)
   sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_5', 'NOQUOTE'), ruleFlowName)
   print(json)
   return
end
local in_version=sas.symgetc("version")
if in_version == nil then
--    sas.print("%1zNo rule flow version provided.")   
   sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_6', 'NOQUOTE'   ))
   return
end
local versions= JSON:decode(json)
if #versions.deployments < 1 then
--    sas.print("%1zFound the rule flow <%s> but it is not deployed." , ruleFlowName )
   sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_7', 'NOQUOTE') , ruleFlowName )
   return;
end
-- default version
local version = versions.deployments[1].version;
for i, v in ipairs(versions.deployments) do
   if tonumber(v.version) == tonumber(in_version) then 
      version = tostring(in_version)
--       sas.print("%3zUsing rule flow " .. ruleFlowName .. " using version " .. in_version .. ".")
      sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_8', 'NOQUOTE') ,ruleFlowName,version)
      break 
   end
end
if (tonumber(version) ~= tonumber(in_version)) then  
--     sas.print("%2zInvalid required version <%s> for rule flow " .. ruleFlowName .. " using current version " .. version .. ".", in_version)
    sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_9', 'NOQUOTE'),tostring(in_version),ruleFlowName,version)
end 
 -- print(version)
local prog
 pass, prog = rest.request( "get", "ruleFlows/".. ruleFlowId .."/deployments/".. version .."/code", nil , "Accept:application/vnd.sas.businessrules.deployments+json;version=1")
if not pass then
--    sas.print("%1zCan't get deployed code for the rule flow:%s.", ruleFlowName)
   sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_10', 'NOQUOTE'), ruleFlowName)
   print(prog)
   return
end
local time_to_get_rules=sas.time()
-- sas.print("%3zTime to acquire rules from BRM was %d seconds.", time_to_get_rules - begintime)
sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_11', 'NOQUOTE'), time_to_get_rules - begintime)
local meth = string.match(prog,"method execute%((.*)method executeDF%(")
meth = string.match(meth,"%);(.*)")
prog = string.gsub(meth,"recordRuleFired(.-);", "")
Debug(prog)
sas.submit("data &dest(compress=yes); set &source; do; " .. prog .. " run;")
-- sas.print("%3zRule flow execution completed.")
sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_12', 'NOQUOTE'))
local time_to_run_rules=sas.time()
-- sas.print("%3zTime to run rules was %d seconds.", time_to_run_rules - time_to_get_rules )
sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_brm_query_version_13', 'NOQUOTE'), time_to_run_rules - time_to_get_rules )
