---
--    Copyright (c) 2016 by SAS Institute Inc., Cary, NC, USA.
-- 
--- Allocation routines for RFW
--  
--  
-- @module rfw_allocation
local M={}
require 'risk.common.global_init'
--- Check for the existence of a variable in a data set
-- @param dsname Fully qualified name of the data set
-- @param var_name Name of the variable to look for
-- @return returns boolean indicating whether the variable exists or not
function M.var_exists(dsname, var_name)
    local rc=false
    --print(ds)
    local dsid = sas.open(dsname,"i")
    if dsid == 0 then
--        sas.print("%1zData set %s doesn't exist.", dsname);
       sas.print(sas.sasmsg('sashelp.stressluamsg',  'sst_var_exists_1', 'NOQUOTE'), dsname)
    else
       local varnum = sas.varnum(dsid, var_name)
       --print("varnum for " .. tostring(var_name) .. " is " .. tostring(varnum))
       rc = varnum ~= 0  -- the variable exists if told the varnum is not zero
       sas.close(dsid)
    end
    --print("var_exists() is returning " .. tostring(rc))
    return rc
end
---Allocation - Apply changes equally across this classification set 
--  @param ccvs The list of ccv variables
--  @param old A table containing the previous summary row
--  @param diff difference in sums
--  @return code snippet
function M.apply_equal(ccvs, old, diff)
    -- make conditional statement
    local cond_stmt = ""
    local cls = string.split(ccvs," ")
    for i, class in ipairs(cls) do
        local esc_str = string.gsub(old[string.lower(class)], "'","''")
        cond_stmt = cond_stmt .. class .. "= '" .. esc_str .. "' "
        if #cls ~= i then
           cond_stmt = cond_stmt .. "and "
        end
    end
    
    local inc = diff/ old._freq_
    
    local snippet = string.resolve([[             
          if @cond_stmt@ then do;
              value = value + @inc@;
          end; 
    ]])
    
    return snippet
end
--- Allocation - Apply changes relative to the value across this classification set 
--  @param ccvs The list of ccv variables
--  @param old A table containing the previous summary row
--  @param diff difference in sums
--  @return code snippet
function M.apply_relative(ccvs, old, diff)
-- make conditional statement
    local cond_stmt = ""
    local cls = string.split(ccvs," ")
    for i, class in ipairs(cls) do
        local esc_str = string.gsub(old[string.lower(class)], "'","''")
        cond_stmt = cond_stmt ..class .. "= '" .. esc_str .. "' "
        if #cls ~= i then
           cond_stmt = cond_stmt .. "and "
        end
    end
    
    local inc = ( 1 + diff/old.value)
    
    local snippet = string.resolve([[       
          if @cond_stmt@ then do;
              value = value *  @inc@;
          end;        
    ]])
    
    return snippet
end
return M
