/* set up source libref  */
libname LoadData;
%unquote(&libRefDef);

%let copydata=true;

/* If the allprice dataset has not been created yet - create it */
/* NOTE: This is drilldown sample specific code and is not in general something one
   would have to do at this stage. The allprice dataset is created if the script
   upload_drilldown_sample.sas is used to create the drilldown sample, however if
   the drilldown sample is loaded from an archive, the detail data will not be present */
/* set force to true in this macro to force the creation of allprice even if it already exists */
%macro create_detail_data(force = false);
   %if %lowcase(&force.) eq true %then %do;
      %let dsid = 0;
   %end;
   %else %do;
      /* Check to see if the allprice dataset exists */
      %let dsid = %sysfunc(open(LoadData.allprice,I));
   %end;
   %if &dsid eq 0 %then %do;
      %let path = %sysfunc(pathname(LoadData));
      %include "&path./allprice.txt";
   %end;   
%mend create_detail_data;
%create_detail_data( );
   
/* Load the data */
proc lua restart;
submit;

local rfw = require 'risk.stress.rfw_api'

rfw.rootdir_set()
rfw.url_set()

local workgroup = sas.symget("workgroup")
local inputdata = "loadData." .. sas.symget("DataSetName")
local success = true
local datakind = sas.symget("DetailKind")
local projectId = sas.symgetn("sst_current_project_id")
local entity = sas.symget("entity")
local scenario_name = sas.symget("scenario_name")

local librefdef, loadid
librefdef = sas.symget("librefdef")

if workgroup == "" then  workgroup = 'Public' end
success, loadid = rfw.detail_data_put(projectId, inputdata, nil, workgroup, datakind, librefdef, entity, scenario_name)  


  if not success then
   sas.print("%1zLoading of detail data %s failed.", inputdata)
  else
   sas.print("%3zLoading of detail data %s succeeded. LoadId = %d", inputdata, loadid)
  end

endsubmit;
run;
