%let as_of_date = 2016-08-01;

/* Increase log_level for more detailed logging. Recommended level is 3 or 4.  5 includes trace option in lua which makes for a VERY large log */
%let log_level = 1;

/* Change any of these to true if you would like any of the related obsjects to be overwritten. For example, if update_scripts is true and a script called 'Drilldown' with version '3.1' already exists, it will be replaced. In all cases name and version need to match for an update to take place. If update is true and the onject does not exist it will be created normally. If update is false and the object with that name and version already exists, it will be left as is */
%let update_scripts = false;
%let update_model_groups = false;
%let update_worksheets = false;

/*
options
   metaserver = "<metadata_server>"
   metaport   = <port>
   metauser   = <metadata_user>
   metapass   = <password>
   ;
*/
%let samp_dir=;
%macro set_samp_dir();
   %let sashome_dir = %sysget(SASHOME);
   %if %index(WIN, &sysscp.) > 0 %then %do;
      %let samp_dir = &sashome_dir.%str(/SASFoundation/9.4/stresssvr/sasmisc/sample_code/drilldown_sample);
   %end;
   %else %do;
      %let samp_dir = &sashome_dir.%str(/SASFoundation/9.4/misc/stresssvr/sample_code/drilldown_sample);
   %end;
%mend;
%set_samp_dir();
libname loadData "&samp_dir./data";
%include "&samp_dir./data/allprice.txt";

proc lua restart;
submit;

   local update_scripts = (string.lower(sas.symget("update_scripts")) == "true")
   local update_model_groups = (string.lower(sas.symget("update_model_groups")) == "true")
   local update_worksheets = (string.lower(sas.symget("update_worksheets")) == "true")

   local rfw        = require 'risk.stress.rfw_api'
   local utils      = require 'risk.common.rest_utils'
   local content    = require 'risk.stress.rfw_content_utils'

   local file_sep = utils.get_file_sep()
   local samp_dir = sas.symget('samp_dir')
   local as_of_date  = sas.symget('as_of_date')

   -- Set the url for rest calls
   local pass = rfw.url_set()
   if not pass then sas.print("%1zUnable to set the rest url.") return end

   ------------------------------------------------------------------------------
   --                            UPLOAD HIERARCHIES                            --
   ------------------------------------------------------------------------------
   --Drilldown
   local hier_list = utils.read_file(samp_dir..file_sep..'hierarchies'..file_sep..'drilldown.txt')
   hier_list = string.split(hier_list, '\n')
   for i, filename in ipairs(hier_list) do
      local hier_name = sas.tranwrd(filename,'.xlsx','')
      hier_name = string.trim(hier_name)
      filename = string.gsub(filename, "\n", "")
      filename = string.trim(filename)
      sas.print("%3zCreating Hierarchy "..hier_name.." in category Drillldown with version 3.1 from the file "..samp_dir..file_sep..'Hierarchy'..file_sep..filename)
      local hpass, hout_str, hcode = rfw.hierarchy_create('Drilldown', hier_name, "For drilldown sample project",samp_dir..file_sep..'hierarchies'..file_sep..filename,"3.1")
      if not hpass then
         sas.print("%1z "..(utils.read_fileref('out') or ""))
         return
      end
   end

    --Time
   local hier_list = utils.read_file(samp_dir..file_sep..'hierarchies'..file_sep..'time.txt')
   hier_list = string.split(hier_list, '\n')
   for i, filename in ipairs(hier_list) do
      hier_name = sas.tranwrd(filename,'.xlsx','')
      hier_name = string.trim(hier_name)
      filename = string.gsub(filename, "\n", "")
      filename = string.trim(filename)
      sas.print("%3zCreating Hierarchy "..hier_name.." in category Time with version 3.1 from the file "..samp_dir..file_sep..'Hierarchy'..file_sep..filename)
      local hpass, hout_str, hcode = rfw.hierarchy_create('Time', hier_name, "For drilldown sample project",samp_dir..file_sep..'hierarchies'..file_sep..filename,"3.1")
      if not hpass then
         sas.print("%1z "..(utils.read_fileref('out') or ""))
         return
      end      
   end
   --Entity
   local hier_list = utils.read_file(samp_dir..file_sep..'hierarchies'..file_sep..'entity.txt')
   hier_list = string.split(hier_list, '\n')
   for i, filename in ipairs(hier_list) do
      hier_name = sas.tranwrd(filename,'.xlsx','')
      hier_name = string.trim(hier_name)
      filename = string.gsub(filename, "\n", "")
      filename = string.trim(filename)
      sas.print("%3zCreating Hierarchy "..hier_name.." in category Entity with version 2.1.3 from the file "..samp_dir..file_sep..'Hierarchy'..file_sep..filename)
      local hpass, hout_str, hcode = rfw.hierarchy_create('Entity', hier_name, "For drilldown sample project",samp_dir..file_sep..'hierarchies'..file_sep..filename, "3.1")
      if not hpass then   
         sas.print("%1z "..(utils.read_fileref('out') or ""))
         return
      end      
   end 

   
   ------------------------------------------------------------------------------
   --                                   SCRIPTS                                --
   ------------------------------------------------------------------------------
   sas.print("%3z Loading the Scripts")
   content.script_load_from_file(samp_dir..file_sep..'scripts'..file_sep..'Scripts.xlsx',update_scripts)

   
   ------------------------------------------------------------------------------
   --                                MODEL GROUPS                              --
   ------------------------------------------------------------------------------
   sas.print("%3z Loading the Model Groups")
   content.model_group_load_from_file(samp_dir..file_sep..'modelgroups'..file_sep..'Model_Groups.xlsx',update_model_groups)


   ------------------------------------------------------------------------------
   --                                 TEMLPATES                                --
   ------------------------------------------------------------------------------
   sas.print("%3z Loading the worksheet templates")
   content.worksheet_batch_load(samp_dir..file_sep..'worksheets',update_worksheets)


   ------------------------------------------------------------------------------
   --                                PROCESSES                                 --
   ------------------------------------------------------------------------------
   sas.print("%3zCreating the sample drilldown process")
   local process_key = rfw.process_key_get("Drilldown_Sample_31")
   if not process_key then
      local reason
      reason, pass = rfw.process_create(samp_dir..file_sep.."process"..file_sep.."Drilldown_Sample_31.xlsx")
      if not pass then
         sas.print("%1z"..tostring(reason))
      end
   end

   ------------------------------------------------------------------------------
   --                                PROJECTS                                  --
   ------------------------------------------------------------------------------
   sas.print("%3zCreating the Drilldown Sample Project") 
   local proj_key = rfw.project_key_get("Drilldown Sample 3.1")
   if not proj_key then
      local worksheets_to_include = {[1]={name = "summary",version = "3.1"}}
      rfw.project_create("Drilldown Sample 3.1", "Sample project for using drilldown with detail data", "Drilldown_Sample_31", as_of_date,'0',worksheets_to_include)
   end

endsubmit;
run;
